#include <conio.h>
#include <dos.h>
#include <stdio.h>

unsigned int BASE=0x300;
void CtrMode(unsigned addr, char cntr, char mode)
{
  int ctrl;
  ctrl = (cntr << 6) | 0x30 | (mode << 1);
  outportb(addr+3,ctrl);
}


unsigned AskFor(char *Prompt,unsigned int OldOne)
{
	char msg[8];
	unsigned int NewOne = 0, Success = 0, Dummy;
	int AddrInputPosX, AddrInputPosY;

	printf("Please enter the %s (in hex)",Prompt);
	printf("or press ENTER for %X.\n>",OldOne);
	AddrInputPosX = wherex(); AddrInputPosY = wherey();
	do {
		gotoxy(AddrInputPosX, AddrInputPosY);
		clreol();
		msg[0] = 5; msg[1] = 0;
		cgets(msg);
		sscanf(msg + 2, "%x", &NewOne);
      Success = 1;
		Dummy = NewOne;
    if (msg[1] == 0) {
			gotoxy(AddrInputPosX, AddrInputPosY);
			printf("%X", OldOne);
			Success = 1;
			Dummy = OldOne;
		}
	} while(!Success);
	return (Dummy);
} /* end of AskForBaseAddress */

void InitCardForAnyTest(void)
{
  outportb(BASE+0x1b,0xFF);  //perform a complete card reset
  outportb(BASE+0x06,0xF0);  //select Channels 0-F
  outport(BASE+2,0x0000);    //Select GAIN CODE "0" for all channels
  outport(BASE+4,0x0000);    //Select GAIN CODE "0" for all channels
  CtrMode(BASE+8,1,1);
}

#define ADCS_SOFTWARE 0x00 // A/D Conversion Start - by Software
#define ADCS_TIMER 0x01    // A/D Conversion Start - by Timer
#define ADCS_EXTERNAL 0x02 // A/D Conversion Start - by External
#define ADCS_RISING 0x00   // A/D Conversion Start - on Rising Edges
#define ADCS_FALLING 0x08  // A/D Conversion Start - on Falling Edges
#define ADCS_SINGLE 0x00   // A/D Conversion Start - Single Conversions
#define ADCS_SCAN 0x04     // A/D Conversion Start - Scan worth of Conversions

void Test(char* msg, short ADCTriggerType)
{
  printf("%s -- Writing %02x to Base + 0x11   ",msg,ADCTriggerType);

  InitCardForAnyTest();
  outportb(BASE+0x11,ADCTriggerType); //configure ADC trigger

  switch(ADCTriggerType & (ADCS_SOFTWARE|ADCS_TIMER|ADCS_EXTERNAL))
  {
  case ADCS_SOFTWARE:
       outportb(BASE+0x01,0);//start software conversion. edge is undetectable
       break;
  case ADCS_TIMER:
       //prepare counter to generate correct edge
       if (ADCTriggerType & ADCS_FALLING)
         CtrMode(BASE+0x08,2,1);
       else
         CtrMode(BASE+0x08,2,0);
       //clear out any spurious data generated by that process
       outportb(BASE+0x1b,0x01);//reset FIFO
       inportb(BASE+0x12);
       delay(1);
       inportb(BASE+0x12);

       //confirm FIFO is empty
       if ((inportb(BASE+0x12) & 0xE0) != 0xC0) puts("FIFO ERROR #1");

       //start a conversion using the right edge
       if (ADCTriggerType & ADCS_FALLING)
         CtrMode(BASE+0x08,2,0);
       else
         CtrMode(BASE+0x08,2,1);
       break;
  case ADCS_EXTERNAL:
       //prepare counter to generate correct edge
       if (ADCTriggerType & ADCS_FALLING)
         CtrMode(BASE+0x08,0,1);
       else
         CtrMode(BASE+0x08,0,0);
       //clear out any spurious data generated by that process
       outportb(BASE+0x1b,0x01);//reset FIFO
       inportb(BASE+0x12);
       delay(1);
       inportb(BASE+0x12);

       //confirm FIFO is empty
       if ((inportb(BASE+0x12) & 0xE0) != 0xC0) puts("FIFO ERROR #1");

       //start a conversion using the right edge
       if (ADCTriggerType & ADCS_FALLING)
         CtrMode(BASE+0x08,0,0);
       else
         CtrMode(BASE+0x08,0,1);
       break;

       break;
  default:
       break;
  } //end switch

  delay(1);
  //confirm FIFO is no longer empty
  if ((inportb(BASE+0x12) & 0xE0) == 0xC0)
  { textcolor(RED);
    gotoxy(70,wherey());cputs("FAILED\n\r");
  }
  else
  { textcolor(GREEN);
    gotoxy(70,wherey());cputs("PASSED\n\r");
  }
  textcolor(LIGHTGRAY);
}



void main (void)
{ int i;

  clrscr();
  printf("This program will Test Start Conversion Types on the 104-AIO16A family\n");
  BASE=AskFor("Base Address",BASE);
  clrscr();
  printf("Base Address is %04x\n",BASE);

  /*
	 This program is going to just run down a long list of tests without
	 pausing, all just kinda jammed into MAIN here.

	 We're going to try to keep it very readable.
  */
  Test("Software Single Rising Edge",ADCS_SOFTWARE|ADCS_SINGLE|ADCS_RISING);
  Test("Software Single Falling Edge",ADCS_SOFTWARE|ADCS_SINGLE|ADCS_FALLING);
  Test("Software Scan Rising Edge",ADCS_SOFTWARE|ADCS_RISING|ADCS_SCAN);
  Test("Software Scan Falling Edge",ADCS_SOFTWARE|ADCS_SCAN|ADCS_FALLING);
  Test("Timer Single Rising Edge",ADCS_TIMER|ADCS_RISING|ADCS_SINGLE);
  Test("Timer Single Falling Edge",ADCS_TIMER|ADCS_FALLING|ADCS_SINGLE);
  Test("Timer Scan Rising Edge",ADCS_TIMER|ADCS_SCAN|ADCS_RISING);
  Test("Timer Scan Falling Edge",ADCS_TIMER|ADCS_SCAN|ADCS_FALLING);
  puts("");
  puts("The EXTERNAL START tests require CTR0 OUT be tied to EXT START TRIG");
  puts("Press any key when ready.");
  getch();
  Test("External Single Rising Edge",ADCS_EXTERNAL|ADCS_SINGLE|ADCS_RISING);
  Test("External Single Falling Edge",ADCS_EXTERNAL|ADCS_SINGLE|ADCS_FALLING);
  Test("External Scan Rising Edge",ADCS_EXTERNAL|ADCS_SCAN|ADCS_RISING);
  Test("External Scan Falling Edge",ADCS_EXTERNAL|ADCS_SCAN|ADCS_FALLING);

  puts("press any key to exit.");
  getch();
}